"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const createError = require("@fastify/error");
const fp = require("fastify-plugin");
const zod = require("zod");
const _interopDefaultCompat = (e) => e && typeof e === "object" && "default" in e ? e : { default: e };
const createError__default = /* @__PURE__ */ _interopDefaultCompat(createError);
const fp__default = /* @__PURE__ */ _interopDefaultCompat(fp);
function isZodType(schema, typeName) {
  var _a;
  return ((_a = schema == null ? void 0 : schema._def) == null ? void 0 : _a.typeName) === typeName;
}
const FST_ZOD_QUERY_COERCION_ERROR = createError__default.default("FST_ZOD_QUERY_COERCION_ERROR", "%s");
const FASTIFY_ZOD_QUERY_COERCION_PROCESSED = Symbol("FASTIFY_ZOD_QUERY_COERCION_PROCESSED");
const isValidNumber = (str) => {
  return !isNaN(Number(str)) && isFinite(Number(str));
};
function transformSchema(schema) {
  if (isZodType(schema, "ZodEffects")) {
    schema._def.schema = transformSchema(schema._def.schema);
    return schema;
  }
  if (isZodType(schema, "ZodBoolean")) {
    return zod.z.preprocess((val) => {
      if (val === "true" || val === "1") return true;
      if (val === "false" || val === "0") return false;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodNumber")) {
    return zod.z.preprocess((val) => {
      if (typeof val === "string" && isValidNumber(val)) return Number(val);
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodString")) {
    return zod.z.preprocess((val) => {
      if (typeof val === "string") return val;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodNull")) {
    return zod.z.preprocess((val) => {
      if (val === "null") return null;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodNullable")) {
    schema._def.innerType = transformSchema(schema._def.innerType);
    return zod.z.preprocess((val) => {
      if (val === "null") return null;
      return val;
    }, schema);
  }
  if (isZodType(schema, "ZodOptional")) {
    schema._def.innerType = transformSchema(schema._def.innerType);
    return schema;
  }
  if (isZodType(schema, "ZodDefault")) {
    schema._def.innerType = transformSchema(schema._def.innerType);
    return schema;
  }
  if (isZodType(schema, "ZodArray")) {
    schema._def.type = transformSchema(schema._def.type);
    return zod.z.preprocess((val) => {
      if (Array.isArray(val)) return val;
      return [val];
    }, schema);
  }
  if (isZodType(schema, "ZodUnion")) {
    schema._def.options = schema._def.options.map(transformSchema);
    return schema;
  }
  if (isZodType(schema, "ZodTuple")) {
    schema._def.items = schema._def.items.map(transformSchema);
    if (schema._def.rest) {
      schema._def.rest = transformSchema(schema._def.rest);
    }
    return zod.z.preprocess((val) => {
      if (Array.isArray(val)) return val;
      return [val];
    }, schema);
  }
  throw new FST_ZOD_QUERY_COERCION_ERROR(`Unsupported schema type for query coercion: ${schema.constructor.name}`);
}
const plugin = async (fastify) => {
  fastify.addHook("onRoute", (route) => {
    var _a;
    if (isZodType((_a = route.schema) == null ? void 0 : _a.querystring, "ZodObject") && !route.schema.querystring[FASTIFY_ZOD_QUERY_COERCION_PROCESSED]) {
      const newShape = Object.entries(route.schema.querystring.shape).reduce((acc, [key, value]) => {
        acc[key] = transformSchema(value);
        return acc;
      }, {});
      route.schema.querystring._def.shape = () => newShape;
      route.schema.querystring[FASTIFY_ZOD_QUERY_COERCION_PROCESSED] = true;
    }
  });
};
const fastifyZodQueryCoercion = fp__default.default(plugin, {
  fastify: ">=4",
  name: "fastify-zod-query-coercion"
});
exports.FASTIFY_ZOD_QUERY_COERCION_PROCESSED = FASTIFY_ZOD_QUERY_COERCION_PROCESSED;
exports.FST_ZOD_QUERY_COERCION_ERROR = FST_ZOD_QUERY_COERCION_ERROR;
exports.default = fastifyZodQueryCoercion;
exports.fastifyZodQueryCoercion = fastifyZodQueryCoercion;
